#include <stdio.h>
#include <ctype.h>

#define CANNOT_OPEN_FILE -1
#define NO_FILE_GIVEN -2
#define BAD_FILE_CONTENTS -3
#define BAD_BASE -4

/*
 * Η συνάρτηση επιστρέφει τον ακέραιο που είναι
 * γραμμένος μέσα στο αρχείο που δίνεται.
 *
 * Υποστηρίζουμε μόνο θετικό ακέραιο αριθμό.
 *
 * Η βάση του συστήματος για το οποίο γίνεται η
 * μετατροπή δίνεται ως 2η παράμετρος (ακέραιος)
 * από 2 έως 10.
 *
 * Εάν κάτι δεν πάει καλά επιστρέφεται αρνητικός
 * αριθμός και για να είναι πιο επεξηγηματικό
 * ορίζουμε τις παραπάνω σταθερές (defines)
 */
long file2long(char *filename, int base) {
    // Υποθέτουμε μηδενικό αποτέλεσμα, επειδή
    // πάνω σε αυτό υπολογίζουμε προσθετικά
    // την αξία του αριθμού
    long result = 0;

    // εάν η βάση του συστήματος δεν είναι αποδεκτή
    // τερματίζουμε με τον κατάλληλο κωδικό σφάλματος
    if (base < 2 || base > 10) { return BAD_BASE; }

    // εάν το όνομα του αρχείου δεν δίνεται
    // τερματίζουμε με τον κατάλληλο κωδικό σφάλματος
    if (filename == NULL) { return NO_FILE_GIVEN; }

    // ανοίγουμε το αρχείο για ανάγνωση ως αρχείο κειμένου
    FILE *fin = fopen(filename, "r");
    // εάν δεν ανοίξει επιτυχώς
    if (fin == NULL) {
        // τερματίζουμε με τον κατάλληλο κωδικό σφάλματος
        return CANNOT_OPEN_FILE;
    }

    // δημιουργούμε μία μεταβλητή που θα κρατά το εκάστοτε
    // "τρέχον" ψηφίο
    char digit;
    // διαβάζοντας κάθε χαρακτήρα στη μεταβλητή digit και
    // για όσο υπάρχουν ακόμα χαρακτήρες να διαβαστούν
    // στο αρχείο
    while( (digit = fgetc(fin)) != EOF ) {
        // εάν το ψηφίο ΔΕΝ είναι από 0 μέχρι έναν πριν τη βάση
        if (digit < '0' || digit >= base + '0') {
            // κλείνουμε το αρχείο και
            fclose(fin);
            // τερματίζουμε με τον κατάλληλο κωδικό σφάλματος
            return BAD_FILE_CONTENTS;
        }

        // η αξία του ψηφίου βρίσκεται εάν αφαιρέσουμε από τον
        // κωδικό του ψηφίου τον κωδικό του 0
        int value = digit - '0';

        // και η νέα τιμή βρίσκεται εάν πολλαπλασιάσουμε την
        // μέχρι τώρα τιμή με τη βάση του συστήματος (οπότε
        // το ψηφίο των μονάδων μηδενίζεται) και προσθέτοντας
        // την αξία του ψηφίου ως "μονάδες"
        result = result * base + value;
    }

    // κλείνουμε το αρχείο και
    fclose(fin);
    // επιστρέφουμε το αριθμητικό αποτέλεσμα
    return result;
}

// απλή εφαρμογή της παραπάνω συνάρτησης
int main() {

    // καλούμε την file2long για το αρχείο με όνομα sample2.txt
    // που βρίσκεται μέσα στον φάκελο extrafiles, ο οποίος με
    // τη σειρά του είναι μέσα στον φάκελο του project (άρα
    // ένα φάκελο πάνω (..) από αυτόν που "ζει" το εκτελέσιμο).
    //
    // ζητάμε δυαδικό σύστημα
    long x = file2long("../extrafiles/sample2.txt", 2);

    // το αποτέλεσμα το συγκρίνουμε ως προς τις πιθανές
    // τιμές λάθους
    switch (x) {
        case CANNOT_OPEN_FILE:
            printf("...");
            return x;
        case NO_FILE_GIVEN:
            printf("...");
            return x;
        case BAD_FILE_CONTENTS:
            printf("...");
            return x;
        case BAD_BASE:
            printf("...");
            return x;
        default:
            // αλλιώς όλα πάνε καλά και συνεχίζει η εκτέλεση
            // πιο κάτω - θα μπορούσε να είναι εδώ κατευθείαν
            // η εκτύπωση του αποτελέσματος
            break;
    }

    // εκτυπώνουμε την τιμή που είχε δώσει η συνάρτηση
    printf("Diabasa : %ld\n", x);
    // ολοκληρώνεται επιτυχώς η εκτέλεση
    return 0;
}
