#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 * Η συνάρτηση splitAt χωρίζει ένα δεδομένο κείμενο (το text) σε
 * μικρότερα κείμενα ("λέξεις") βάσει ενός χαρακτήρα διαχωρισμού
 * (separator) π.χ. στο κείμενο "one,two,three" με διαχωριστικό
 * το ',' θα επιστρέφει έναν πίνακα ["one","two","three"]
 *
 * Προσοχή! Όταν λέμε "λέξη" εννοούμε το οποιοδήποτε κείμενα,
 * πριν, μετά και ανάμεσα στους χαρακτήρες διαχωρισμού (separator)
 *
 * Το πλήθος των "λέξεων" που βρήκε, δηλαδή το μήκος του πίνακα
 * που επιστράφηκε δίνεται μέσω της παραμέτρου N, η οποία γι'αυτό
 * το λόγο δίνεται ως pointer.
 *
 * Εάν δεν είναι δυνατόν να ολοκληρωθεί επιτυχώς η εκτέλεση της
 * συνάρτησης θα πρέπει να επιστρέφει NULL.
 */
char **splitAt( char separator, char *text, int *N ) {
    // αρχικά γίνεται έλεγχος των δεδομένων. Εάν οι pointers είναι
    // NULL, τότε δεν μπορούμε να προχωρήσουμε
    if (text == NULL || N == NULL) {
        return NULL;
    }

    // υπολογίζω το μήκος του δεδομένου κειμένου
    int length = strlen(text);
    // ζητάω μνήμη για να αποθηκεύσω ένα αντίγραφο του κειμένου, δηλαδή
    // όσους χαρακτήρες έχει (length) και έναν επιπλέον (+1) για το
    // τερματικό μηδέν στο τέλος του κειμένου
    char *copy = (char *)malloc(length + 1);
    // Εάν δεν υπάρχει διαθέσιμη μνήμη
    if (copy == NULL) {
        // τότε η συνάρτηση αποτυγχάνει και δεν υπάρχει κάτι που
        // χρειάζεται αποδέσμευση (πχ μνήμη ή αρχείο)
        return NULL;
    }
    // αντιγράφεται το αρχικό κείμενο στον χώρο που κρατήσαμε για
    // το αντίγραφο
    strcpy(copy, text);

    // υποθέτω αρχικό πλήθος λέξεων 1 (αφού τόσες "λέξεις") θα
    // υπάρχουν ακόμα και όταν δεν υπάρχει καμία εμφάνιση του
    // χαρακτήρα separator
    int wordCount = 1;
    // για κάθε γράμμα του κειμένου
    for (int i = 0; i < length; ++i) {
        // εάν ο χαρακτήρας του αντιγράφου στη θέση i είναι ο
        // διαχωριστικός (separator)
        if (copy[i] == separator) {
            // τότε πρέπει να υπάρχει μία ακόμα λέξη (που ξεκινά
            // μετά από αυτόν τον χαρακτήρα
            wordCount++;
            // βάζω τον τερματικό κωδικό 0 για την προηγούμενη λέξη
            copy[i] = 0;
        }
    }

    // βάσει του αριθμού των λέξεων δεσμεύουμε μνήμη για τους
    // pointers σε κάθε λέξη
    char **result = (char **)malloc(wordCount * sizeof(char *));
    // εάν δεν υπάρχει αρκετή μνήμη
    if (result == NULL) {
        // αποδεσμεύουμε αυτή που είχαμε δεσμεύσει
        free(copy);
        // και επιστρέφουμε NULL ως ένδειξη του προβλήματος
        return NULL;
    }

    // η μεταβλητή αυτή δείχνει ποιά είναι η "τρέχουσα" λέξη
    // αυτή δηλαδή που έχουμε βρει πιο "πρόσφατα"
    int currentWord = 0;
    // σε αυτή τη θέση βάζουμε και τον δείκτη για την αρχή της
    // λέξης. Η πρώτη λέξη είναι πάντα στην αρχή του κειμένου
    result[currentWord] = copy;
    // κατόπιν για κάθε χαρακτήρα
    for (int i = 0; i < length; ++i) {
        // εάν ο χαρακτήρας είναι ο 0, σημαίνει ότι εκεί τελειώνει η
        // τρέχουσα λέξη (επειδή σε αυτό το σημείο υπήρχε ένας
        // χαρακτήρας διαχωρισμού)
        if (copy[i] == 0) {
            // αυξάνουμε κατά ένα την τρέχουσα λέξη (μπαίνουμε στην επόμενη)
            currentWord++;
            // και η αρχή της είναι στην επόμενη θέση χαρακτήρα (i+1) ενώ
            // με το & παίρνουμε τον pointer γι'αυτή τη θέση.
            result[currentWord] = &copy[i+1];
            // εάν η τρέχουσα λέξη που βρήκαμε είναι η τελευταία
            // (και επειδη ως δείκτης πίνακα πάει μέχρι την προηγούμενη
            // τιμή από το πλήθος των στοιχείων, χρειάζεται και το -1)
            if (currentWord == wordCount - 1) {
                // τότε σταματούν οι επαναλήψεις
                break;
            }
        }
    }

    // θέτουμε στην μεταβλητή που επιστρέφουμε το πλήθος των "λέξεων" που
    // βρέθηκαν κατά την παραπάνω διαδικασία
    *N = wordCount;
    // και επιστρέφουμε τον πίνακα με τους pointers στις "λέξεις"
    return result;
}

/*
 * Υπόδειγμα χρήσης της συνάρτησης
 */
int main() {
    // η μεταβλητή που θα κρατήσει το πλήθος των "λέξεων"
    int n;
    // καλείται η συνάρτηση για separator , και
    // για κείμενο ",,one,two,,three,,"
    char **words = splitAt(',', ",,one,two,,three,,", &n);

    // εάν δεν βρεθούν λέξεις
    if (words == NULL) {
        // τυπώνουμε ένα μήνυμα με την έκλπηξή μας
        printf("OOPS!\n\n");
        // και τερματίζουμε τον κώδικα με μη-μηδενικό
        // κωδικό
        return -1;
    }

    // εάν όλα πήγαν καλά για κάθε "λέξη" που βρέθηκε,
    for (int i = 0; i < n; ++i) {
        // εκτυπώνουμε την α/α της και την ίδια τη "λέξη"
        printf("Lexi #%d : %s\n", i+1, words[i]);
    }

    // τερματισμός του προγράμματος με μηδενικό κωδικό σφάλματος
    return 0;
}
